
# q-expansion of newform 192.8.f.b, downloaded from the LMFDB on 07 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 192
weight = 8
poly_data = [1, 0, -1, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 2, 0, -1], 1], [[-4, 0, 8, 0], 1], [[0, 0, 0, 4], 1]]

hecke_ring_character_values = [[127, [-1, 0, 0, 0]], [133, [-1, 0, 0, 0]], [65, [-1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, -27, 0, 0], [0, 0, 0, 0], [0, 0, 0, -127], [0, 0, 0, 0], [0, 0, 883, 0], [0, 0, 0, 0], [0, -9578, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, -44729], [0, 0, 79253, 0], [0, 0, 0, 0], [0, -72078, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, -38429, 0], [0, 2833994, 0, 0], [0, 0, 0, 0], [6274810, 0, 0, 0], [0, 0, 0, -2190761], [0, 0, 0, 0], [0, 0, 0, 0], [12245198, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, -2006555], [0, 0, 0, 0], [0, 0, 3055623, 0], [0, 0, 0, 0], [0, 0, 0, -11281405], [0, 0, 0, 0], [0, 0, 0, 0], [0, 17788670, 0, 0], [0, 0, 0, 0], [0, 0, 0, 19259729], [0, 0, -7470921, 0], [0, 39139142, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, -22301371, 0], [0, 0, 0, 0], [-96518014, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 36196153], [0, -92778854, 0, 0], [0, 0, 0, 65724239], [0, 0, 0, 0], [0, 0, -43690623, 0], [0, 0, 0, 0], [0, 0, 0, 0], [-57587054, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 121389023], [0, 0, -102111279, 0], [0, 0, 0, 0], [0, 356450478, 0, 0], [0, 0, 0, 0], [0, 576206586, 0, 0], [0, 0, 0, 0], [1084883206, 0, 0, 0], [0, 0, 0, 0], [0, 247733630, 0, 0], [1275706622, 0, 0, 0], [0, 0, 0, 0], [0, 0, 206115215, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, -316499273], [0, 0, -275676283, 0], [0, 149194562, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 231651879, 0], [0, 0, 0, 0], [2127270058, 0, 0, 0], [0, 0, 0, 0], [0, 0, -301802135, 0], [0, 0, 0, 0], [1115656414, 0, 0, 0], [0, 0, 0, 812885353], [0, 0, 0, 0], [0, 0, 0, 0], [-4067093510, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 255682355], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 1086514517], [0, 0, 0, 0], [0, -2463203718, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, -1260205102, 0, 0], [0, 0, 920315971, 0], [0, -2756052922, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, -3751839026, 0, 0], [-4239333458, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [7416536426, 0, 0, 0], [0, 0, 0, -590327005], [0, 0, -241744979, 0], [0, 0, 0, 0], [0, -1202000162, 0, 0], [0, 0, 0, -2925423259], [0, 0, 0, 0], [0, 7760471526, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 1918796665, 0], [4999175950, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, -4814286090, 0, 0], [0, 0, 0, 0], [0, 0, 2704290253, 0], [0, 0, 0, 0], [0, 0, 0, -2003988283], [0, 0, -1152756489, 0], [0, -6296403990, 0, 0], [0, 0, 0, 0], [0, 0, 0, -3648931637], [0, 0, 3288251869, 0], [0, 0, 0, 0], [-6300532222, 0, 0, 0], [0, 0, 0, 0], [0, -6617682794, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, -16826188686, 0, 0], [0, 0, 0, 0], [0, 0, 0, 7965566141], [0, 0, 0, 0], [0, 0, -4646346385, 0], [0, 0, 0, 0], [0, 0, -3869246683, 0], [0, 0, 0, 0], [0, 18519482530, 0, 0], [0, 0, 0, 0], [0, 0, -2415865669, 0], [0, 0, 0, 0], [0, 13834141238, 0, 0], [0, 0, 0, 0], [0, -9420118318, 0, 0], [0, 0, 0, 0], [0, 0, 0, 9442884893], [0, 0, 0, 0], [3585136438, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 10560795205], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, -11675393741], [0, 0, 3991244157, 0]]
